@@ -16,9 +16,11 @@ use serde::{Deserialize, Serialize};
 use serde_json::{json, Value};
 
 use btc::BtcAddress;
+use constants::ELECTRUM_DEFAULT_EXPIRATION;
 use error::Result;
 
 pub mod btc;
+mod constants;
 pub mod error;
 pub mod ext;
 
@@ -38,6 +40,9 @@ enum ElectrumMethod {
     GetAddressHistory,
     GetAddressBalance,
 
+    #[serde(rename = "onchain_history")]
+    GetOnchainHistory,
+
     #[serde(rename = "list_wallets")]
     ListWallets,
 
@@ -89,12 +94,14 @@ enum Param {
 
     Password,
     Fee,
+    FeeRate,
     Outputs,
     Amount,
     Memo,
     Pending,
     Expired,
     Paid,
+    Expiration,
 }
 
 struct JsonRpcBodyBuilder {
@@ -411,6 +418,7 @@ impl Electrum {
         destination: &BtcAddress<'a>,
         amount: Decimal,
         fee: Option<Decimal>,
+        fee_rate: Option<Decimal>,
     ) -> Result<Response<Body>> {
         let mut builder = JsonRpcBody::new()
             .method(ElectrumMethod::PayTo)
@@ -421,6 +429,10 @@ impl Electrum {
             builder = builder.add_param(Param::Fee, Value::from(fee.to_string()));
         }
 
+        if let Some(fee_rate) = fee_rate {
+            builder = builder.add_param(Param::FeeRate, Value::from(fee_rate.to_string()));
+        }
+
         self.call_method(&builder.build()).await
     }
 
@@ -458,7 +470,12 @@ impl Electrum {
     /// The address will be considered as used after this operation.
     /// If no payment is received, the address will be considered as unused
     /// if the payment request is deleted from the wallet.
-    pub async fn add_request(&self, amount: Decimal, memo: Option<&str>) -> Result<Response<Body>> {
+    pub async fn add_request(
+        &self,
+        amount: Decimal,
+        memo: Option<&str>,
+        expiration: Option<u64>,
+    ) -> Result<Response<Body>> {
         let amount = amount.to_string();
 
         let mut builder = JsonRpcBody::new()
@@ -469,6 +486,9 @@ impl Electrum {
             builder = builder.add_param(Param::Memo, Value::from(memo))
         };
 
+        let expiration = expiration.unwrap_or(ELECTRUM_DEFAULT_EXPIRATION);
+        builder = builder.add_param(Param::Expiration, Value::from(expiration));
+
         self.call_method(&builder.build()).await
     }
 
@@ -514,6 +534,18 @@ impl Electrum {
         )
         .await
     }
+
+    /// Wallet onchain history.
+    /// Returns the transaction history of your wallet.
+    pub async fn get_onchain_history(&self) -> Result<Response<Body>> {
+        self.call_method(
+            JsonRpcBody::new()
+                .method(ElectrumMethod::GetOnchainHistory)
+                .build()
+                .borrow(),
+        )
+        .await
+    }
 }
 
 #[cfg(test)]
